#pragma once

#include "geos/operation/buffer/BufferOp.h"
#include "geos/operation/buffer/BufferParameters.h"
#include "geos/operation/distance/DistanceOp.h"
#include "geos/operation/overlay/OverlayOp.h"
#include "geos/operation/valid/IsValidOp.h"
#include "geos/algorithm/BoundaryNodeRule.h"
#include "geos/operation/relate/RelateOp.h"
#include "geos/operation/linemerge/LineMerger.h"
#include "geos/operation/linemerge/LineSequencer.h"
