#pragma once

#include <xs.h>
#include <xs/export.h>

#include "geom.h"
#include "io.h"
#include "prep.h"
#include "algorithm.h"
#include "noding.h"
#include "index.h"
#include "precision.h"
#include "operation.h"
#include "triangulate.h"
