package Geo::Geos::GeometryFactory;

use Geo::Geos;
use Geo::Geos::Geometry;
use Geo::Geos::Point;
use Geo::Geos::GeometryCollection;
use Geo::Geos::MultiPoint;
use Geo::Geos::LineString;
use Geo::Geos::MultiLineString;
use Geo::Geos::LinearRing;
use Geo::Geos::Polygon;
use Geo::Geos::MultiPolygon;

1;
